!----------------------------------------------------------------------
! File: LinDisDim.f90
! (Linear Dispersion Dimensional Form)
!
! Description: Solve for steady states of Camassa's film equation, then continue
! along families of traveling wave solutions.
!
! Author: Nathan Vaughn
! University of North Carolina
! 8/31/15
!
!----------------------------------------------------------------------

SUBROUTINE FUNC(NDIM,R,ICP,PAR,IJAC,F,DFDR,DFDP)
!     ---------- ----
IMPLICIT NONE
INTEGER, INTENT(IN) :: NDIM, ICP(*), IJAC
DOUBLE PRECISION, INTENT(IN) :: R(NDIM), PAR(*)
DOUBLE PRECISION, INTENT(OUT) :: F(NDIM)
DOUBLE PRECISION, INTENT(INOUT) :: DFDR(NDIM,NDIM), DFDP(NDIM,*)
DOUBLE PRECISION X0,X1,X2,c,K,h0,B,a,pi,g,rho,gamma,mu

X0=R(1)
X1=R(2)
X2=R(3)
B=PAR(1) ! Dispersion coeficient
c=PAR(2) ! wave speed
K=PAR(3) ! constant of integration
a=PAR(4) ! radius


pi =  4d0 * atan(1d0)
g = 980d0;        ! cm/s^2
rho = 0.97d0;     ! g/cm^3
gamma = 21.5d0;   ! g/s^2
mu = 129d0;


F(1)=X1
F(2)=X2
F(3)= (g*rho - (gamma*X1)/X0**2 + &
      (16*K - a**4*g*rho + 8*c*mu*X0**2 + 16*B*X2)/ &
      (a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a)))/gamma

if (IJAC == 0) return
DFDR = 0d0

DFDR(1,2) = 1d0

DFDR(2,3) = 1d0

DFDR(3,1) = ((2*gamma*X1)/X0**3 + (8*X0* &
            (16*K - a**4*g*rho + 8*c*mu*X0**2 + 16*B*X2)* &
            (a**2 - X0**2 + 2*X0**2*Log(X0/a)))/ &
            (a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a))**2 + &
            (16*c*mu*X0)/(a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a)))/ &
            gamma
DFDR(3,2) = -X0**(-2)
DFDR(3,3) = (16*B)/(gamma*(a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a)))

if (IJAC == 1) return
DFDP(1:NDIM,1:4) = 0d0

DFDP(3,1) = (16*X2)/(gamma*(a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a)))
DFDP(3,2) =  (8*mu*X0**2)/(gamma*(a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a)))
DFDP(3,3) = 16/(gamma*(a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a)))
DFDP(3,4) = (8*((a - X0)*(a + X0)*(a**4*g*rho*X0**2 - &
            4*a**2*(2*K + c*mu*X0**2 + 2*B*X2) + &
            4*X0**2*(2*K + c*mu*X0**2 + 2*B*X2)) + &
            2*a**4*g*rho*X0**4*Log(X0/a)))/ &
            (a*gamma*(a**4 - 4*a**2*X0**2 + 3*X0**4 - 4*X0**4*Log(X0/a))**2)

END SUBROUTINE FUNC

!----------------------------------------------------------------------

SUBROUTINE STPNT(NDIM,R,PAR,T)
!     ---------- -----
IMPLICIT NONE
INTEGER, INTENT(IN) :: NDIM
DOUBLE PRECISION, INTENT(INOUT) :: R(NDIM),PAR(*)
DOUBLE PRECISION, INTENT(IN) :: T
DOUBLE PRECISION mu,g,rho

g = 980d0;        ! cm/s^2
rho = 0.97d0;     ! g/cm^3
mu = 129d0;

!R(1) = 0.3488091d0;
R(1) = 0.35d0;
R(2) = 0;
R(3) = 0;

! p = [ B, c, K, a ]
PAR(4)=0.5d0;
PAR(1)=-0.1d0
PAR(2)=0.15d0
PAR(3)=(-8*PAR(2)*mu*R(1)**2 + 4*PAR(4)**2*g*rho*R(1)**2 - 3*g*rho*R(1)**4 + &
       4*g*rho*R(1)**4*Log(R(1)/PAR(4)))/16.



END SUBROUTINE STPNT

!----------------------------------------------------------------------
SUBROUTINE BCND


END SUBROUTINE BCND

!----------------------------------------------------------------------
SUBROUTINE ICND



END SUBROUTINE ICND

SUBROUTINE FOPT
END SUBROUTINE FOPT

SUBROUTINE PVLS(NDIM,U,PAR)
!     ---------- ----

IMPLICIT NONE
INTEGER, INTENT(IN) :: NDIM
DOUBLE PRECISION, INTENT(IN) :: U(NDIM)
DOUBLE PRECISION, INTENT(INOUT) :: PAR(*)

DOUBLE PRECISION, EXTERNAL :: GETP,GETU2
INTEGER NDX,NCOL,NTST

PAR(5)=PAR(4)-GETP('INT',1,U)
PAR(6)=PAR(4)-GETP('MIN',1,U)


END SUBROUTINE PVLS
!----------------------------------------------------------------------






